@@ -6,7 +6,8 @@ Distances computation and comparison of kapture objects
 
 import math
 import numpy as np
-from typing import List, Optional, Tuple, Any, Union
+from typing import Any, Dict, List, Optional, Set, Tuple
+import inspect
 
 import kapture
 from kapture import flatten
@@ -27,9 +28,10 @@ def float_iszero(distance: float, threshold: float = 1e-05) -> bool:
     return math.isclose(distance, 0.0, rel_tol=threshold, abs_tol=threshold)
 
 
-def is_distance_within_threshold(pose_distance: Tuple[float, float],
-                                 pose_thresholds: Tuple[float, float] = (1e-05, 1e-05)
-                                 ) -> bool:
+def is_distance_within_threshold(
+        pose_distance: Tuple[float, float],
+        pose_thresholds: Tuple[float, float] = (1e-05, 1e-05)
+) -> bool:
     """
     compare a pose distance tuple to (0,0) with some thresholds
 
@@ -42,7 +44,7 @@ def is_distance_within_threshold(pose_distance: Tuple[float, float],
     """
     translation_distance, rotation_distance = pose_distance
     translation_threshold, rotation_threshold = pose_thresholds
-    return float_iszero(translation_distance, translation_threshold) \
+    return float_iszero(translation_distance, translation_threshold)\
         and float_iszero(rotation_distance, rotation_threshold)
 
 
@@ -114,9 +116,11 @@ def equal_sensors(sensors_a: Optional[kapture.Sensors], sensors_b: Optional[kapt
             return False
 
         equal_params = False
-        if sensor_a.sensor_type == 'camera':
+        if sensor_a.sensor_type in kapture.ALL_CAMERA_SENSOR_TYPES:
             assert isinstance(sensor_a, Camera)
             assert isinstance(sensor_b, Camera)
+            if sensor_a.sensor_type != sensor_b.sensor_type:
+                return False
             if sensor_a.camera_type == sensor_b.camera_type:
                 equal_params = equal_camera_params(sensor_a.camera_params, sensor_b.camera_params)
         else:
@@ -165,8 +169,9 @@ def equal_rigs(rigs_a: Optional[kapture.Rigs], rigs_b: Optional[kapture.Rigs]) -
     return True
 
 
-def equal_trajectories(trajectories_a: Optional[kapture.Trajectories],
-                       trajectories_b: Optional[kapture.Trajectories]) -> bool:
+def equal_trajectories(
+        trajectories_a: Optional[kapture.Trajectories],
+        trajectories_b: Optional[kapture.Trajectories]) -> bool:
     """
     Compare two instances of kapture.Trajectories.
     Poses are compared with is_distance_within_threshold(pose_transform_distance())
@@ -222,12 +227,18 @@ def log_difference(a: List[Tuple[Any, ...]], b: List[Tuple[Any, ...]], func_name
         getLogger().debug('{}:\n{}'.format(func_name, '\n'.join(diffs)))
 
 
-def equal_nested_dict_or_set(data_a, data_b, name_to_log) -> bool:
+def equal_nested_dict_or_set(data_a, data_b, name_to_log, expected_type=None) -> bool:
     """
     Compare two instances of dictionary or set
 
     :return: True if they are identical, False otherwise.
     """
+    if expected_type is not None:  # do type checking
+        for data_x in [data_a, data_b]:
+            if data_x is not None and not isinstance(data_x, expected_type):
+                raise TypeError(f'expecting type {expected_type} in {name_to_log} (got {type(data_x)})')
+
+    # early check if one (or both) are None
     if data_a is None and data_b is None:
         return True
     elif data_a is None and data_b is not None:
@@ -235,6 +246,7 @@ def equal_nested_dict_or_set(data_a, data_b, name_to_log) -> bool:
     elif data_a is not None and data_b is None:
         return False
 
+    # check values
     flattened_a = list(flatten(data_a, is_sorted=True))
     flattened_b = list(flatten(data_b, is_sorted=True))
     are_equal = (flattened_a == flattened_b)
@@ -243,16 +255,42 @@ def equal_nested_dict_or_set(data_a, data_b, name_to_log) -> bool:
     return are_equal
 
 
-def equal_image_features(data_a: Optional[Union[kapture.Keypoints, kapture.Descriptors, kapture.GlobalFeatures]],
-                         data_b: Optional[Union[kapture.Keypoints, kapture.Descriptors, kapture.GlobalFeatures]]
-                         ) -> bool:
+def equal_sets(data_a: Set[Any], data_b: Set[Any], func_name: str = 'equal_sets'):
+    set_difference = data_a.difference(data_b)
+    are_equal = len(set_difference) == 0
+    if not are_equal:
+        getLogger().debug('{}:\n{}'.format(func_name, '\n'.join((str(s) for s in set_difference))))
+    return are_equal
+
+
+def equal_keypoints(
+        data_a: kapture.Keypoints,
+        data_b: kapture.Keypoints
+) -> bool:
     """
-    Compare two instances of kapture features (keypoints, descriptors or global features).
+    Compare two instances of kapture keypoints.
 
     :param data_a: first set of features
     :param data_b: second set of features
     :return: True if they are identical, False otherwise.
     """
+    assert isinstance(data_a, kapture.Keypoints)
+    assert isinstance(data_b, kapture.Keypoints)
+    if data_a.type_name != data_b.type_name or data_a.dtype != data_b.dtype or data_a.dsize != data_b.dsize:
+        return False
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_sets(data_a, data_b, current_function_name)
+
+
+def equal_keypoints_collections(data_a: Optional[Dict[str, kapture.Keypoints]],
+                                data_b: Optional[Dict[str, kapture.Keypoints]]):
+    """
+    Compare two keypoints collections.
+    :param data_a: first collection of features
+    :param data_b: second collection of features
+    :return: True if they are identical, False otherwise.
+    """
+    # early check if one is None
     if data_a is None and data_b is None:
         return True
     elif data_a is None and data_b is not None:
@@ -264,18 +302,116 @@ def equal_image_features(data_a: Optional[Union[kapture.Keypoints, kapture.Descr
     assert data_a is not None
     assert data_b is not None
 
-    if data_a.type_name != data_b.type_name or data_a.dtype != data_b.dtype or data_a.dsize != data_b.dsize:
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    if not equal_sets(set(data_a.keys()), set(data_b.keys()), current_function_name):
         return False
-    flattened_a = list(flatten(data_a, is_sorted=True))
-    flattened_b = list(flatten(data_b, is_sorted=True))
-    are_equal = (flattened_a == flattened_b)
-    if not are_equal:
-        log_difference(flattened_a, flattened_b, 'equal_image_features')
-    return are_equal
+    for keypoint_type in data_a.keys():
+        if not equal_keypoints(data_a[keypoint_type], data_b[keypoint_type]):
+            return False
+    return True
 
 
-def equal_records_camera(records_a: Optional[kapture.RecordsCamera],
-                         records_b: Optional[kapture.RecordsCamera]) -> bool:
+def equal_descriptors(
+        data_a: kapture.Descriptors,
+        data_b: kapture.Descriptors
+) -> bool:
+    """
+    Compare two instances of kapture descriptors.
+
+    :param data_a: first set of features
+    :param data_b: second set of features
+    :return: True if they are identical, False otherwise.
+    """
+    assert isinstance(data_a, kapture.Descriptors)
+    assert isinstance(data_b, kapture.Descriptors)
+    if data_a.type_name != data_b.type_name or data_a.dtype != data_b.dtype or data_a.dsize != data_b.dsize \
+            or data_a.keypoints_type != data_b.keypoints_type or data_a.metric_type != data_b.metric_type:
+        return False
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_sets(data_a, data_b, current_function_name)
+
+
+def equal_descriptors_collections(data_a: Optional[Dict[str, kapture.Descriptors]],
+                                  data_b: Optional[Dict[str, kapture.Descriptors]]):
+    """
+    Compare two descriptors collections.
+    :param data_a: first collection of features
+    :param data_b: second collection of features
+    :return: True if they are identical, False otherwise.
+    """
+    # early check if one is None
+    if data_a is None and data_b is None:
+        return True
+    elif data_a is None and data_b is not None:
+        return False
+    elif data_a is not None and data_b is None:
+        return False
+
+    # should not happen because of previous lines, use assert to help your ide figure out the type of data
+    assert data_a is not None
+    assert data_b is not None
+
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    if not equal_sets(set(data_a.keys()), set(data_b.keys()), current_function_name):
+        return False
+    for descriptors_type in data_a.keys():
+        if not equal_descriptors(data_a[descriptors_type], data_b[descriptors_type]):
+            return False
+    return True
+
+
+def equal_global_features(
+        data_a: kapture.GlobalFeatures,
+        data_b: kapture.GlobalFeatures
+) -> bool:
+    """
+    Compare two instances of kapture global features.
+
+    :param data_a: first set of features
+    :param data_b: second set of features
+    :return: True if they are identical, False otherwise.
+    """
+    assert isinstance(data_a, kapture.GlobalFeatures)
+    assert isinstance(data_b, kapture.GlobalFeatures)
+    if data_a.type_name != data_b.type_name or data_a.dtype != data_b.dtype or data_a.dsize != data_b.dsize \
+            or data_a.metric_type != data_b.metric_type:
+        return False
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_sets(data_a, data_b, current_function_name)
+
+
+def equal_global_features_collections(data_a: Optional[Dict[str, kapture.GlobalFeatures]],
+                                      data_b: Optional[Dict[str, kapture.GlobalFeatures]]):
+    """
+    Compare two global_features collections.
+    :param data_a: first collection of features
+    :param data_b: second collection of features
+    :return: True if they are identical, False otherwise.
+    """
+    # early check if one is None
+    if data_a is None and data_b is None:
+        return True
+    elif data_a is None and data_b is not None:
+        return False
+    elif data_a is not None and data_b is None:
+        return False
+
+    # should not happen because of previous lines, use assert to help your ide figure out the type of data
+    assert data_a is not None
+    assert data_b is not None
+
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    if not equal_sets(set(data_a.keys()), set(data_b.keys()), current_function_name):
+        return False
+    for global_features_type in data_a.keys():
+        if not equal_global_features(data_a[global_features_type], data_b[global_features_type]):
+            return False
+    return True
+
+
+def equal_records_camera(
+        records_a: Optional[kapture.RecordsCamera],
+        records_b: Optional[kapture.RecordsCamera]) -> bool:
     """
     Compare two instances of kapture.RecordsCamera.
 
@@ -283,11 +419,14 @@ def equal_records_camera(records_a: Optional[kapture.RecordsCamera],
     :param records_b: second set of records
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(records_a, records_b, 'equal_records_camera')
+    expected_type = kapture.RecordsCamera
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
 
 
-def equal_records_lidar(records_a: Optional[kapture.RecordsLidar],
-                        records_b: Optional[kapture.RecordsLidar]) -> bool:
+def equal_records_lidar(
+        records_a: Optional[kapture.RecordsLidar],
+        records_b: Optional[kapture.RecordsLidar]) -> bool:
     """
     Compare two instances of kapture.RecordsLidar.
 
@@ -295,11 +434,14 @@ def equal_records_lidar(records_a: Optional[kapture.RecordsLidar],
     :param records_b: second set of records
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(records_a, records_b, 'equal_records_lidar')
+    expected_type = kapture.RecordsLidar
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
 
 
-def equal_records_wifi(records_a: Optional[kapture.RecordsWifi],
-                       records_b: Optional[kapture.RecordsWifi]) -> bool:
+def equal_records_wifi(
+        records_a: Optional[kapture.RecordsWifi],
+        records_b: Optional[kapture.RecordsWifi]) -> bool:
     """
     Compare two instances of kapture.RecordsWifi.
 
@@ -307,11 +449,29 @@ def equal_records_wifi(records_a: Optional[kapture.RecordsWifi],
     :param records_b: second set of records
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(records_a, records_b, 'equal_records_wifi')
+    expected_type = kapture.RecordsWifi
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
 
 
-def equal_records_gnss(records_a: Optional[kapture.RecordsGnss],
-                       records_b: Optional[kapture.RecordsGnss]) -> bool:
+def equal_records_bluetooth(
+        records_a: Optional[kapture.RecordsBluetooth],
+        records_b: Optional[kapture.RecordsBluetooth]) -> bool:
+    """
+    Compare two instances of kapture.RecordsBluetooth.
+
+    :param records_a: first set of records
+    :param records_b: second set of records
+    :return: True if they are identical, False otherwise.
+    """
+    expected_type = kapture.RecordsBluetooth
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
+
+
+def equal_records_gnss(
+        records_a: Optional[kapture.RecordsGnss],
+        records_b: Optional[kapture.RecordsGnss]) -> bool:
     """
     Compare two instances of kapture.RecordsGnss.
 
@@ -319,11 +479,59 @@ def equal_records_gnss(records_a: Optional[kapture.RecordsGnss],
     :param records_b: second set of records
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(records_a, records_b, 'equal_records_gnss')
+    expected_type = kapture.RecordsGnss
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
 
 
-def equal_matches(matches_a: Optional[kapture.Matches],
-                  matches_b: Optional[kapture.Matches]) -> bool:
+def equal_records_accelerometer(
+        records_a: Optional[kapture.RecordsAccelerometer],
+        records_b: Optional[kapture.RecordsAccelerometer]) -> bool:
+    """
+    Compare two instances of kapture.RecordsAccelerometer.
+
+    :param records_a: first set of records
+    :param records_b: second set of records
+    :return: True if they are identical, False otherwise.
+    """
+    expected_type = kapture.RecordsAccelerometer
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
+
+
+def equal_records_gyroscope(
+        records_a: Optional[kapture.RecordsGyroscope],
+        records_b: Optional[kapture.RecordsGyroscope]) -> bool:
+    """
+    Compare two instances of kapture.RecordsGyroscope.
+
+    :param records_a: first set of records
+    :param records_b: second set of records
+    :return: True if they are identical, False otherwise.
+    """
+    expected_type = kapture.RecordsGyroscope
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
+
+
+def equal_records_magnetic(
+        records_a: Optional[kapture.RecordsMagnetic],
+        records_b: Optional[kapture.RecordsMagnetic]) -> bool:
+    """
+    Compare two instances of kapture.RecordsMagnetic.
+
+    :param records_a: first set of records
+    :param records_b: second set of records
+    :return: True if they are identical, False otherwise.
+    """
+    expected_type = kapture.RecordsMagnetic
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(records_a, records_b, current_function_name, expected_type)
+
+
+def equal_matches(
+        matches_a: kapture.Matches,
+        matches_b: kapture.Matches) -> bool:
     """
     Compare two instances of kapture.Matches.
 
@@ -331,11 +539,44 @@ def equal_matches(matches_a: Optional[kapture.Matches],
     :param matches_b: second set of matches
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(matches_a, matches_b, 'equal_matches')
+    assert isinstance(matches_a, kapture.Matches)
+    assert isinstance(matches_b, kapture.Matches)
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_sets(matches_a, matches_b, current_function_name)
 
 
-def equal_observations(data_a: Optional[kapture.Observations],
-                       data_b: Optional[kapture.Observations]) -> bool:
+def equal_matches_collections(data_a: Optional[Dict[str, kapture.Matches]],
+                              data_b: Optional[Dict[str, kapture.Matches]]):
+    """
+    Compare two matches collections.
+    :param data_a: first collection of matches
+    :param data_b: second collection of matches
+    :return: True if they are identical, False otherwise.
+    """
+    # early check if one is None
+    if data_a is None and data_b is None:
+        return True
+    elif data_a is None and data_b is not None:
+        return False
+    elif data_a is not None and data_b is None:
+        return False
+
+    # should not happen because of previous lines, use assert to help your ide figure out the type of data
+    assert data_a is not None
+    assert data_b is not None
+
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    if not equal_sets(set(data_a.keys()), set(data_b.keys()), current_function_name):
+        return False
+    for keypoints_type in data_a.keys():
+        if not equal_matches(data_a[keypoints_type], data_b[keypoints_type]):
+            return False
+    return True
+
+
+def equal_observations(
+        data_a: Optional[kapture.Observations],
+        data_b: Optional[kapture.Observations]) -> bool:
     """
     Compare two instances of kapture.Observations.
 
@@ -343,11 +584,14 @@ def equal_observations(data_a: Optional[kapture.Observations],
     :param data_b: second set of observations
     :return: True if they are identical, False otherwise.
     """
-    return equal_nested_dict_or_set(data_a, data_b, 'equal_observations')
+    expected_type = kapture.Observations
+    current_function_name = inspect.getframeinfo(inspect.currentframe()).function
+    return equal_nested_dict_or_set(data_a, data_b, current_function_name, expected_type)
 
 
-def equal_points3d(points3d_a: Optional[kapture.Points3d],
-                   points3d_b: Optional[kapture.Points3d]) -> bool:
+def equal_points3d(
+        points3d_a: Optional[kapture.Points3d],
+        points3d_b: Optional[kapture.Points3d]) -> bool:
     """
     Compare two instances of kapture.Points3d.
 
@@ -380,7 +624,9 @@ def equal_points3d(points3d_a: Optional[kapture.Points3d],
     return are_equal
 
 
-def equal_kapture(data_a: kapture.Kapture, data_b: kapture.Kapture) -> bool:
+def equal_kapture(
+        data_a: kapture.Kapture,
+        data_b: kapture.Kapture) -> bool:  # noqa: C901
     """
     Compare two instances of Kapture.
      Poses are compared with is_distance_within_threshold(pose_transform_distance())
@@ -402,25 +648,22 @@ def equal_kapture(data_a: kapture.Kapture, data_b: kapture.Kapture) -> bool:
         return False
 
     # compare records
-    if not equal_records_camera(data_a.records_camera, data_b.records_camera):
-        return False
-    if not equal_records_lidar(data_a.records_lidar, data_b.records_lidar):
-        return False
-    if not equal_records_wifi(data_a.records_wifi, data_b.records_wifi):
-        return False
-    if not equal_records_gnss(data_a.records_gnss, data_b.records_gnss):
-        return False
+    for record_sensor in ['camera', 'lidar', 'wifi', 'bluetooth', 'gnss', 'accelerometer', 'gyroscope', 'magnetic']:
+        record_name = f'records_{record_sensor}'
+        equal_record_function = eval(f'equal_records_{record_sensor}')
+        if not equal_record_function(getattr(data_a, record_name), getattr(data_b, record_name)):
+            return False
 
     # compare image features (keypoints, descriptors, global_features)
-    if not equal_image_features(data_a.keypoints, data_b.keypoints):
+    if not equal_keypoints_collections(data_a.keypoints, data_b.keypoints):
         return False
-    if not equal_image_features(data_a.descriptors, data_b.descriptors):
+    if not equal_descriptors_collections(data_a.descriptors, data_b.descriptors):
         return False
-    if not equal_image_features(data_a.global_features, data_b.global_features):
+    if not equal_global_features_collections(data_a.global_features, data_b.global_features):
         return False
 
     # compare matches
-    if not equal_matches(data_a.matches, data_b.matches):
+    if not equal_matches_collections(data_a.matches, data_b.matches):
         return False
 
     # compare observations
