@@ -1,5 +1,6 @@
 # Copyright 2020-present NAVER Corp. Under BSD 3-clause license
-from typing import List, Optional, Type
+
+from typing import List
 from kapture.io.records import TransferAction, import_record_data_from_dir_auto
 from kapture.utils.logging import getLogger
 
@@ -12,8 +13,8 @@ def merge_records_data(image_list: List[List[str]],
     Merge several records data. keep only the first image.
 
     :param image_list: list of image_names
-    :param image dir paths
-    :param directory root path to the merged kapture
+    :param image_paths: list of image paths
+    :param kapture_path: root path to the merged kapture
     :param images_import_method: choose how to import actual image files
     """
     assert len(image_list) > 0
