@@ -10,12 +10,11 @@ import logging
 from typing import Tuple, Dict
 from tqdm import tqdm
 import kapture
-from kapture.io.csv import get_csv_fullpath, records_camera_from_file
+from kapture.io.csv import get_csv_fullpath
 from kapture.io.records import images_to_filepaths
 from warnings import warn
 
 
-
 logger = logging.getLogger('exif')
 
 
@@ -79,8 +78,8 @@ def convert_dms_to_float(
 
     assert all(isinstance(v, tuple) for v in dms)
     decimal = convert_rational_to_float(dms[0]) + \
-              convert_rational_to_float(dms[1]) / 60 + \
-              convert_rational_to_float(dms[2]) / 3600
+        convert_rational_to_float(dms[1]) / 60 + \
+        convert_rational_to_float(dms[2]) / 3600
     # handle South or west coordinates.
     if reference is not None and reference.upper() in ['S', 'W']:
         decimal *= -1
@@ -148,7 +147,7 @@ def extract_gps_from_exif(
     cam_to_gps_id = {  # cam_id -> gps_id
         cam_id: 'GPS_' + cam_id
         for cam_id, sensor in kapture_data.sensors.items()
-        if sensor.sensor_type == 'camera'
+        if sensor.sensor_type == kapture.SENSOR_TYPE_CAMERA
     }  # cam_id -> gps_id
 
     # set all gps to EPSG:4326
@@ -181,11 +180,11 @@ def import_gps_from_exif(
     :param kapture_dirpath: path to kapture directory. kpautre data are modified.
     :return:
     """
-    logger.info(f'loading kapture partial ...')
-    skip_heavy_useless = {kapture.Trajectories,
+    logger.info('loading kapture partial ...')
+    skip_heavy_useless = [kapture.Trajectories,
                           kapture.RecordsLidar, kapture.RecordsWifi,
                           kapture.Keypoints, kapture.Descriptors, kapture.GlobalFeatures,
-                          kapture.Matches, kapture.Points3d, kapture.Observations}
+                          kapture.Matches, kapture.Points3d, kapture.Observations]
     kapture_data = kapture.io.csv.kapture_from_dir(kapture_dirpath, skip_list=skip_heavy_useless)
 
     # load exifs
