@@ -1,8 +1,14 @@
 # Copyright 2020-present NAVER Corp. Under BSD 3-clause license
 
+"""
+Represents a 6 dimensions pose.
+"""
+
+import math
 import numpy as np
 import quaternion
 from typing import List, Optional
+from numba import njit
 
 
 class PoseTransform:
@@ -42,7 +48,7 @@ class PoseTransform:
             self._t = t
         elif isinstance(t, list):
             # its a list: convert to numpy if valid
-            self._t = np.array(t, dtype=np.float)
+            self._t = np.array(t, dtype=float)
         else:
             self._t = None
 
@@ -91,8 +97,24 @@ class PoseTransform:
         assert self._r is not None
         assert self._t is not None
         r_inv = self.r.inverse()
-        t_inv = np.matmul(quaternion.as_rotation_matrix(r_inv), self.t * -1.0)
-        return PoseTransform(r_inv, t_inv)
+        rotation_inv_matrix = np.empty((3, 3), dtype=float)
+        rotation_inv_as_np = np.array([r_inv.w, r_inv.x, r_inv.y, r_inv.z])
+        _as_rotation_matrix_njit(rotation_inv_as_np, rotation_inv_matrix)
+        t_inv = np.matmul(rotation_inv_matrix, self.t * -1.0)
+        pose_inv = PoseTransform.__new__(PoseTransform)
+        pose_inv._r = r_inv
+        pose_inv._t = t_inv
+        return pose_inv
+
+    def rescale(self, scale: float):
+        """
+        rescales translation part
+        :param pose:
+        :param scale:
+        :return: scaled pose
+        """
+        if self._t is not None:
+            self._t = self._t * scale
 
     @staticmethod
     def compose(pose_list: List['PoseTransform']) -> 'PoseTransform':
@@ -106,12 +128,18 @@ class PoseTransform:
         pose_composed = pose_list[0]
         for pose_current in pose_list[1:]:
             # shrink the poses from the left
-            assert pose_current._r is not None
-            assert pose_current._t is not None
+            # assert pose_current._r is not None
+            # assert pose_current._t is not None
             new_r = pose_composed.r * pose_current.r
-            new_t = np.add(np.matmul(quaternion.as_rotation_matrix(
-                pose_composed.r), pose_current.t), pose_composed.t)
-            pose_composed = PoseTransform(new_r, new_t)
+            rotation_matrix = np.empty((3, 3), dtype=float)
+            rotation_composed_as_np = np.array([pose_composed.r.w, pose_composed.r.x,
+                                                pose_composed.r.y, pose_composed.r.z])
+            _as_rotation_matrix_njit(rotation_composed_as_np, rotation_matrix)
+            new_t = np.add(np.matmul(rotation_matrix, pose_current.t), pose_composed.t)
+
+            pose_composed = PoseTransform.__new__(PoseTransform)
+            pose_composed._r = new_r
+            pose_composed._t = new_t
         return pose_composed
 
     def transform_points(self, points3d: np.ndarray) -> np.ndarray:
@@ -128,9 +156,57 @@ class PoseTransform:
         if points3d.shape[1] == 6:  # expunge RGB
             points3d = points3d[:, 0:3]
         points3d = points3d.transpose()
-        rotation_matrix = quaternion.as_rotation_matrix(self.r)
+        rotation_matrix = np.empty((3, 3), dtype=float)
+        rotation_as_np = np.array([self.r.w, self.r.x, self.r.y, self.r.z])
+        _as_rotation_matrix_njit(rotation_as_np, rotation_matrix)
         points3d = np.add(np.matmul(rotation_matrix, points3d), self.t)
         return points3d.transpose()
 
     def __repr__(self) -> str:
         return 'r:{},  t:{}'.format(self.r_raw, self.t_raw)
+
+    def __eq__(self, other) -> bool:
+        if (self._t is None and other.t is not None) or (self._t is not None and other.t is None):
+            return False
+        if (self._r is None and other.r is not None) or (self._r is not None and other.r is None):
+            return False
+        self_trans = self.t_raw
+        other_trans = other.t_raw
+        for left, right in zip(self_trans, other_trans):
+            # Check distances at 10-2 millimeters
+            if not math.isclose(left, right, rel_tol=1.e-05, abs_tol=1.e-05):
+                return False
+        self_rot = self.r_raw
+        other_rot = other.r_raw
+        for left, right in zip(self_rot, other_rot):
+            # Check angles at 10-2 degrees
+            if not math.isclose(left, right, rel_tol=1.e-02, abs_tol=1.e-02):
+                return False
+        return True
+
+
+# https://github.com/moble/quaternion/blob/main/src/quaternion/__init__.py#L200
+@njit
+def _as_rotation_matrix_njit(q, rot_mat):
+    q_norm = q[0]*q[0] + q[1]*q[1] + q[2]*q[2] + q[3]*q[3]
+    if abs(q_norm-1.0) < 1e-14:
+        rot_mat[0, 0] = 1 - 2*(q[2]**2 + q[3]**2)
+        rot_mat[0, 1] = 2*(q[1]*q[2] - q[3]*q[0])
+        rot_mat[0, 2] = 2*(q[1]*q[3] + q[2]*q[0])
+        rot_mat[1, 0] = 2*(q[1]*q[2] + q[3]*q[0])
+        rot_mat[1, 1] = 1 - 2*(q[1]**2 + q[3]**2)
+        rot_mat[1, 2] = 2*(q[2]*q[3] - q[1]*q[0])
+        rot_mat[2, 0] = 2*(q[1]*q[3] - q[2]*q[0])
+        rot_mat[2, 1] = 2*(q[2]*q[3] + q[1]*q[0])
+        rot_mat[2, 2] = 1 - 2*(q[1]**2 + q[2]**2)
+    else:
+        rot_mat[0, 0] = 1 - 2*(q[2]**2 + q[3]**2)/q_norm
+        rot_mat[0, 1] = 2*(q[1]*q[2] - q[3]*q[0])/q_norm
+        rot_mat[0, 2] = 2*(q[1]*q[3] + q[2]*q[0])/q_norm
+        rot_mat[1, 0] = 2*(q[1]*q[2] + q[3]*q[0])/q_norm
+        rot_mat[1, 1] = 1 - 2*(q[1]**2 + q[3]**2)/q_norm
+        rot_mat[1, 2] = 2*(q[2]*q[3] - q[1]*q[0])/q_norm
+        rot_mat[2, 0] = 2*(q[1]*q[3] - q[2]*q[0])/q_norm
+        rot_mat[2, 1] = 2*(q[2]*q[3] + q[1]*q[0])/q_norm
+        rot_mat[2, 2] = 1 - 2*(q[1]**2 + q[2]**2)/q_norm
+    return
