@@ -4,18 +4,15 @@
 This files contains IO operations on Record related data.
 """
 
-from enum import auto
-import logging
 import os
 import os.path as path
-import shutil
-from typing import Tuple, Any, Dict, Type, Optional, Union, Iterable, List
-from tqdm import tqdm
-
+from typing import Dict, Optional, Union, Iterable, Tuple
+import numpy as np
 import kapture
+from kapture.core.Records import RecordsDepth
 from kapture.utils.paths import path_secure
-from kapture.utils.Collections import AutoEnum
 from kapture.utils.logging import getLogger
+from kapture.io.binary import TransferAction, transfer_files_from_dir, array_from_file, array_to_file
 
 logger = getLogger()
 
@@ -23,118 +20,6 @@ logger = getLogger()
 RECORD_DATA_DIRNAME = path_secure(path.join('sensors', 'records_data'))
 
 
-class TransferAction(AutoEnum):
-    """
-    All possible operations when transferring recorded data (eg. images, Wifi, IMU, ...)
-    or computed data (features, keypoints, ...).
-    Transferring means importing, exporting, and more generally going from one representation
-    to another representation of the data.
-    """
-    skip = auto()
-    root_link = auto()
-    copy = auto()
-    move = auto()
-    link_absolute = auto()
-    link_relative = auto()
-
-
-########################################################################################################################
-# transfer files functions
-def transfer_files_from_dir_link(
-        source_filepath_list: Iterable[str],
-        destination_filepath_list: Iterable[str],
-        force_overwrite: bool = False,
-        do_relative_link: bool = False
-) -> None:
-    """
-    Transfer every files by linking given from the source list to destination list.
-    The matching between source and kapture files are explicitly given.
-
-    :param source_filepath_list: input list of source files. Uses guess_filepaths to obtains it from filenames.
-    :param destination_filepath_list: input list of destination files (in kapture tree).
-    :param force_overwrite: if True, overwrite destination file.
-    :param do_relative_link: if True, do relative links else absolute links.
-    """
-    hide_progress_bar = logger.getEffectiveLevel() > logging.INFO
-    for src, dst in tqdm(zip(source_filepath_list, destination_filepath_list), disable=hide_progress_bar):
-        os.makedirs(path.dirname(dst), exist_ok=True)
-        if force_overwrite and path.lexists(dst):
-            os.remove(dst)
-        try:  # on windows, symlink requires some privileges, and may crash if not
-            if do_relative_link:
-                src = path.relpath(src, path.dirname(dst))
-            os.symlink(src, dst)
-        except OSError as e:
-            logger.critical('unable to create symlink on image directory, due to privilege restrictions.')
-            raise e
-
-
-def transfer_files_from_dir_copy(
-        source_filepath_list: Iterable[str],
-        destination_filepath_list: Iterable[str],
-        force_overwrite: bool = False,
-        delete_source: bool = False
-) -> None:
-    """
-    Transfer every files by copying given from the source list to destination list.
-    The matching between source and kapture files are explicitly given.
-    If delete_source is activated, it moves files instead copying them.
-
-    :param source_filepath_list: input list of absolute path to source files.
-    :param destination_filepath_list: input list of absolute path to destination files.
-    :param force_overwrite: if True, overwrite destination file.
-    :param delete_source: if True, delete the imported files from source_record_dirpath.
-    """
-    hide_progress_bar = logger.getEffectiveLevel() > logging.INFO
-    for src, dst in tqdm(zip(source_filepath_list, destination_filepath_list), disable=hide_progress_bar):
-        os.makedirs(path.dirname(dst), exist_ok=True)
-        if force_overwrite and path.lexists(dst):
-            os.remove(dst)
-        if delete_source:
-            shutil.move(src, dst)
-        else:
-            shutil.copyfile(src, dst)
-
-
-def transfer_files_from_dir(
-        source_filepath_list: Iterable[str],
-        destination_filepath_list: Iterable[str],
-        copy_strategy: TransferAction = TransferAction.copy,
-        force_overwrite: bool = False
-) -> None:
-    """
-    Transfer files (copy or link files) from source to destination.
-    The matching between source and kapture files are explicitly given.
-    The actual import can be done using actual file copy (copy/move), or symlinks (absolute/relative).
-
-    :param source_filepath_list: input list of source files.
-    :param destination_filepath_list: input list of destination files (in kapture tree).
-    :param copy_strategy: transfer strategy to apply.
-    :param force_overwrite: if True, overwrite destination file.
-    """
-    if TransferAction.skip == copy_strategy:
-        return
-
-    elif TransferAction.copy == copy_strategy or TransferAction.move == copy_strategy:
-        transfer_files_from_dir_copy(
-            source_filepath_list=source_filepath_list,
-            destination_filepath_list=destination_filepath_list,
-            force_overwrite=force_overwrite,
-            delete_source=(TransferAction.move == copy_strategy)
-        )
-
-    elif TransferAction.link_absolute == copy_strategy or TransferAction.link_relative == copy_strategy:
-        transfer_files_from_dir_link(
-            source_filepath_list=source_filepath_list,
-            destination_filepath_list=destination_filepath_list,
-            force_overwrite=force_overwrite,
-            do_relative_link=(TransferAction.link_relative == copy_strategy)
-        )
-
-    else:
-        raise ValueError(f'Unsupported copy action {copy_strategy}')
-
-
 ########################################################################################################################
 def get_record_fullpath(
         kapture_dirpath: str = '',
@@ -151,7 +36,11 @@ def get_record_fullpath(
 
 
 def records_to_filepaths(
-        records: Union[kapture.RecordsCamera, kapture.RecordsWifi, kapture.RecordsLidar, kapture.RecordsGnss],
+        records: Union[kapture.RecordsCamera,
+                       kapture.RecordsDepth,
+                       kapture.RecordsWifi,
+                       kapture.RecordsLidar,
+                       kapture.RecordsGnss],
         kapture_dirpath: str
 ) -> Dict[str, str]:
     """
@@ -237,7 +126,7 @@ def import_record_data_from_dir_auto(
 def get_image_fullpath(kapture_dir_path: str, image_filename: Optional[str] = None) -> str:
     """
     Get the full path of the image file in the kapture.
-     If image file name is missing, this gives the top directory of the images.
+     If image_filename is missing, this gives the top directory (records_data).
 
     :param kapture_dir_path: the kapture top directory
     :param image_filename: optional image file name
@@ -255,3 +144,58 @@ def images_to_filepaths(images: kapture.RecordsCamera, kapture_dirpath: str) ->
     :return: images name to images file path dictionary
     """
     return records_to_filepaths(images, kapture_dirpath)
+
+
+# depth maps file paths ################################################################################################
+def get_depth_map_fullpath(kapture_dir_path: str, depth_map_filename: Optional[str] = None) -> str:
+    """
+    Get the full path of the depth map file in the kapture.
+     If depth_map_filename is missing, this gives the top directory (records_data).
+
+    :param kapture_dir_path: the kapture top directory
+    :param depth_map_filename: optional image file name
+    :return: Depth map file full path
+    """
+    return get_record_fullpath(kapture_dir_path, depth_map_filename)
+
+
+def depth_maps_to_filepaths(depth_records: kapture.RecordsDepth, kapture_dirpath: str) -> Dict[str, str]:
+    """
+    Computes filepaths for depth maps records.
+
+    :param images: images records
+    :param kapture_dirpath: top kapture directory path
+    :return: images name to images file path dictionary
+    """
+    return records_to_filepaths(depth_records, kapture_dirpath)
+
+
+# depth maps IO ########################################################################################################
+def depth_map_from_file(filepath: str, size: Tuple[int, int]) -> np.array:
+    """
+    Load the depth map from binary file to a numpy array.
+
+    :param filepath: path to the file
+    :param size: [width, height]
+    :return: the depth map as a numpy array
+    """
+    assert isinstance(size, tuple) and len(size) == 2
+    dtype = kapture.RecordsDepth.dtype
+    dsize = int(size[0] * size[1])
+    bitmap = array_from_file(filepath, dtype, dsize)
+    bitmap = bitmap.reshape((size[1], size[0]))
+    return bitmap
+
+
+def depth_map_to_file(filepath: str, depth_map: np.array) -> None:
+    """
+    Writes the depth map from numpy array to binary file.
+
+    :param filepath: file path
+    :param depth_map: depth map as a numpy array
+    """
+    # ensure depth maps are expected type RecordsDepth.dtype (aka float32)
+    if not depth_map.dtype == RecordsDepth.dtype:
+        logger.warning('depth maps are converted to float')
+        depth_map = depth_map.astype(RecordsDepth.dtype)
+    array_to_file(filepath, depth_map)
