@@ -7,10 +7,9 @@ Export the list of images of a kapture into a file listing the image files, and
 
 import logging
 import os
-import os.path as path
 import argparse
 # kapture
-import path_to_kapture
+import path_to_kapture  # noqa: F401
 import kapture
 import kapture.utils.logging
 from kapture.io.csv import kapture_from_dir
@@ -29,10 +28,16 @@ def export_image_list(kapture_path: str, output_path: str, export_camera_params:
     :param export_camera_params: if True, add camera parameters after every file name
     :param force: Silently overwrite image list file if already exists.
     """
-    os.makedirs(os.path.dirname(output_path), exist_ok=True)
+    os.makedirs(os.path.dirname(os.path.abspath(output_path)), exist_ok=True)
     safe_remove_file(output_path, force)
-
-    kapture_to_export = kapture_from_dir(kapture_path)
+    skip_heavy_useless = [kapture.Trajectories,
+                          kapture.RecordsLidar, kapture.RecordsWifi,
+                          kapture.RecordsAccelerometer, kapture.RecordBluetooth,
+                          kapture.RecordGnss, kapture.RecordGyroscope,
+                          kapture.RecordsDepth, kapture.RecordsMagnetic,
+                          kapture.Keypoints, kapture.Descriptors, kapture.GlobalFeatures,
+                          kapture.Matches, kapture.Points3d, kapture.Observations]
+    kapture_to_export = kapture_from_dir(kapture_path, skip_list=skip_heavy_useless)
     output_content = []
     logger.info('starting conversion...')
     for _, sensor_id, filename in kapture.flatten(kapture_to_export.records_camera, is_sorted=True):
